@@ -46,6 +46,8 @@
 #include "seahorse-ssh-key.h"
 #include "seahorse-ssh-operation.h"
 #include "seahorse-passphrase.h"
+#include "seahorse-util.h"
+#include "seahorse-unknown-source.h"
 
 #ifndef DEBUG_SSHAGENT_ENABLE
 #if _DEBUG
@@ -61,6 +63,9 @@
 #define DEBUG_MSG(x)
 #endif
 
+static GSList *ssh_agent_cached_fingerprints = NULL;
+static gboolean ssh_agent_fingerprints_loaded = FALSE;
+
 static gboolean ssh_agent_initialized = FALSE;
 static GList *ssh_agent_connections = NULL;
 
@@ -89,10 +94,11 @@ typedef struct _SSHProxyConn {
     do { (cp)[0] = (value) >> 8; (cp)[1] = (value); } while (0)
 
 /* The various messages we're interested in */
-#define SSH_AGENTC_REQUEST_RSA_IDENTITIES   1
-#define SSH_AGENT_RSA_IDENTITIES_ANSWER     2
 #define SSH2_AGENTC_REQUEST_IDENTITIES      11
 #define SSH2_AGENT_IDENTITIES_ANSWER        12
+#define SSH2_AGENTC_ADD_IDENTITY            17
+#define SSH2_AGENTC_REMOVE_IDENTITY         18
+#define SSH2_AGENTC_REMOVE_ALL_IDENTITIES   19
     
 struct _SSHMsgHeader {
     guchar msgid;
@@ -110,6 +116,53 @@ typedef struct _SSHMsgNumIdentities SSHMsgNumIdentities;
  * IMPLEMENTATION
  */
 
+static void
+clear_cached_fingerprints()
+{
+    DEBUG_MSG (("Clearing our internal cache of loaded SSH fingerprints\n"));
+    seahorse_util_string_slist_free (ssh_agent_cached_fingerprints);
+    ssh_agent_cached_fingerprints = NULL;
+    ssh_agent_fingerprints_loaded = FALSE;
+}
+
+static gboolean
+parse_cached_fingerprint (SeahorseSSHKeyData *keydata, gpointer arg)
+{
+    ssh_agent_cached_fingerprints = 
+        g_slist_prepend (ssh_agent_cached_fingerprints, g_strdup (keydata->fingerprint));
+    seahorse_ssh_key_data_free (keydata);
+    return TRUE;
+}
+
+static void
+load_cached_fingerprints ()
+{
+    SeahorseKeySource *sksrc;
+    gchar *output;
+    GError *err = NULL;
+    
+    if (ssh_agent_fingerprints_loaded)
+        return;
+    
+    clear_cached_fingerprints ();
+    
+    DEBUG_MSG (("Loading internal cache of fingerprints\n"));
+    ssh_agent_fingerprints_loaded = TRUE;
+    
+    sksrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_SSH, SKEY_LOC_LOCAL);
+    g_return_if_fail (SEAHORSE_IS_SSH_SOURCE (sksrc));
+    
+    /* TODO: This could possibly done more efficiently by talking with the agent ourselves */
+    output = seahorse_ssh_operation_sync (SEAHORSE_SSH_SOURCE (sksrc), "/bin/sh -c \"" SSH_ADD_PATH " -L; true\"", &err);
+    if (!output) {
+        g_warning ("couldn't list keys in SSH agent: %s", err && err->message ? err->message : "");
+        return;
+    }
+    
+    seahorse_ssh_key_data_parse (output, parse_cached_fingerprint, NULL, NULL);
+    g_free (output);
+}
+
 /* Free the given connection structure */
 static void
 free_connection (SSHProxyConn *cn)
@@ -147,50 +200,54 @@ free_connections (gpointer data, gpointer user_data)
     free_connection ((SSHProxyConn*)data);
 }
 
-static gint
-read_ssh_message (GIOChannel* source, gchar *buf, gsize bufsize)
+static gchar*
+read_ssh_message (GIOChannel* source, gsize* length)
 {
     GError *err = NULL;
+    gchar buf[256];
     gsize bytes;
-    guint length, read, r;
+    GString *msg = NULL;
+    guint r;
     
-    g_assert (bufsize >= 4);
+    *length = 0;
     
-    /* First read the length of the response packet. */
-    g_io_channel_read_chars (source, buf, 4, &bytes, &err);
+    /* First read the length of the response packet */
+    g_io_channel_read_chars (source, (gchar*)buf, 4, &bytes, &err);
     
     if (err != NULL || bytes != 4) {
         if(bytes == 0)
-            return -1;
+            return NULL;
         g_critical ("couldn't read length from socket: %s", 
                     err && err->message ? err->message : "");
         g_clear_error (&err);
-        return -1;
+        return NULL;
     }
     
-    length = GET_32BIT (buf);
+    *length = GET_32BIT (buf);
+    msg = g_string_sized_new (1024);
     
-    /* To prevent buffer overflow discard anything over bufsize */
-    for (read = 0; read < length; read += bufsize) {
+    while (msg->len < *length) {
         
-        r = length > bufsize ? bufsize : length;
+        r = (*length - msg->len) > sizeof (buf) ? sizeof (buf) : (*length - msg->len);
         
         /* Now read the actual msg */
-        g_io_channel_read_chars (source, buf, r, &bytes, &err);
+        g_io_channel_read_chars (source, (gchar*)buf, r, &bytes, &err);
         
         if (err != NULL || bytes != r) {
-            g_critical ("couldn't read from socket: %s", 
-                         err && err->message ? err->message : "");
+            g_critical ("couldn't read from socket: %s (%d/%d/%d)", 
+                         err && err->message ? err->message : "", r, bytes, *length);
             g_clear_error (&err);
-            return -1;
+            g_string_free (msg, TRUE);
+            return NULL;
         }
         
-        read += r;
-        length -= r;
+        g_string_append_len (msg, buf, r);
     }
+    
+    g_assert (*length == msg->len);
 
-    DEBUG_MSG (("received message: %d (len: %d)\n", (int)(guchar)buf[0], bytes));    
-    return bytes;
+    DEBUG_MSG (("received message: %d (len: %d)\n", (int)(gchar)msg->str[0], *length));
+    return (gchar*)g_string_free (msg, FALSE);
 }
 
 static gboolean
@@ -210,7 +267,7 @@ write_ssh_message (GIOChannel* source, gchar *buf, gsize bufsize)
         return FALSE;
     }
     
-    g_io_channel_write_chars (source, buf, bufsize, &written, &err);
+    g_io_channel_write_chars (source, (gchar*)buf, bufsize, &written, &err);
     
     if (err != NULL || written != bufsize) {
         g_critical ("couldn't write to socket: %s", 
@@ -219,92 +276,90 @@ write_ssh_message (GIOChannel* source, gchar *buf, gsize bufsize)
         return FALSE;
     }
 
-    DEBUG_MSG (("sent message: %d (len: %d)\n", (int)(guchar)buf[0], bufsize));
+    DEBUG_MSG (("sent message: %d (len: %d)\n", (int)(gchar)buf[0], bufsize));
     return TRUE;
 }
 
-static guint
-get_num_identities (SSHProxyConn *cn, gboolean version1)
+static gint
+get_num_identities (SSHProxyConn *cn)
 {
-    gchar buf[1024];
-    SSHMsgHeader *req;
+    gchar *msg;
+    SSHMsgHeader req;
     SSHMsgNumIdentities *resp;
     guint length;
+    guint ret = -1;
     
-    memset (buf, 0, sizeof (buf));
+    memset (&req, 0, sizeof (req));
+    req.msgid = SSH2_AGENTC_REQUEST_IDENTITIES;
     
-    req = (SSHMsgHeader*)buf;
-    req->msgid = version1 ? SSH_AGENTC_REQUEST_RSA_IDENTITIES : 
-                            SSH2_AGENTC_REQUEST_IDENTITIES;
-    
-    if (!write_ssh_message (cn->outchan, buf, sizeof (*req)))
+    if (!write_ssh_message (cn->outchan, (gchar*)(&req), sizeof (req)))
         return -1;
     
-    length = read_ssh_message (cn->outchan, buf, sizeof (buf));
+    msg = read_ssh_message (cn->outchan, &length);
+    if (!msg)
+        return -1;
     
-    resp = (SSHMsgNumIdentities*)buf;
+    resp = (SSHMsgNumIdentities*)msg;
     if (length >= sizeof (*resp)) {
-        
-        if ((version1 && resp->head.msgid == SSH_AGENT_RSA_IDENTITIES_ANSWER) || 
-            (!version1 && resp->head.msgid == SSH2_AGENT_IDENTITIES_ANSWER)) {
-            return resp->num_identities;
-        }
+        if (resp->head.msgid == SSH2_AGENT_IDENTITIES_ANSWER) 
+            ret = resp->num_identities;
+    } else {
+        g_warning ("can't understand SSH agent protocol");
     }
     
-    g_warning ("can't understand SSH agent protocol");
-    return -1;
+    g_free (msg);
+    return ret;
 }
 
 static gboolean 
-filter_version_one (SeahorseKey *key, gpointer data)
-{
-    g_return_val_if_fail (SEAHORSE_IS_SSH_KEY (key), FALSE);
-    return seahorse_ssh_key_get_algo (SEAHORSE_SSH_KEY (key)) == SSH_ALGO_RSA1;
-}
-
-static gboolean 
-filter_version_two (SeahorseKey *key, gpointer data)
+filter_keys (SeahorseKey *key, gpointer data)
 {
     guint algo;
     g_return_val_if_fail (SEAHORSE_IS_SSH_KEY (key), FALSE);
     algo = seahorse_ssh_key_get_algo (SEAHORSE_SSH_KEY (key));
-    return algo == SSH_ALGO_RSA || algo == SSH_ALGO_DSA;
+    return (algo == SSH_ALGO_RSA || algo == SSH_ALGO_DSA) && 
+            seahorse_key_get_etype (key) == SKEY_PRIVATE;
 }
 
 static GList*
-find_ssh_keys (gboolean version1)
+find_ssh_keys ()
 {
     SeahorseKeyPredicate skp;
     
     memset (&skp, 0, sizeof (skp));
     skp.ktype = SKEY_SSH;
     skp.location = SKEY_LOC_LOCAL;
-    skp.custom = version1 ? filter_version_one : filter_version_two;
+    skp.custom = filter_keys;
     
     return seahorse_context_find_keys_full (SCTX_APP (), &skp);
 }
 
+static gboolean
+update_status (gpointer dummy)
+{
+    seahorse_agent_status_update ();
+    return FALSE;
+}
+
 static void
 load_ssh_key (SeahorseSSHKey *skey)
 {
     SeahorseOperation *op;
     SeahorseSSHSource *ssrc;
-    gchar *cmd;
     
-    DEBUG_MSG (("Loading SSH key: %s\n", seahorse_ssh_key_get_filename (skey, TRUE)));
-    
-    cmd = g_strdup_printf (SSH_ADD_PATH " '%s'", seahorse_ssh_key_get_filename (skey, TRUE));
+    DEBUG_MSG (("Loading SSH key: %s\n", seahorse_ssh_key_get_location (skey)));
     ssrc = SEAHORSE_SSH_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (skey)));
-    op = seahorse_ssh_operation_new (ssrc, cmd, NULL, 0, _("SSH Passphrase"), skey);
-    
-    g_free (cmd);
     
+    op = seahorse_ssh_operation_agent_load (ssrc, skey);
     seahorse_operation_wait (op);
     
     if (!seahorse_operation_is_successful (op))
         g_warning ("couldn't run ssh-add to add a key identity: %s", 
                    seahorse_operation_get_error (op)->message);
     
+    clear_cached_fingerprints ();
+    g_timeout_add (500, update_status, NULL);
+
     g_object_unref (op);
 }
 
@@ -312,7 +367,6 @@ static gboolean
 process_message (SSHProxyConn *cn, gboolean from_client, gchar *msg, gsize len)
 {
     SSHMsgHeader *smsg = (SSHMsgHeader*)msg;
-    gboolean version1 = FALSE;
     GList *keys, *l;
     guint count;
     
@@ -323,15 +377,20 @@ process_message (SSHProxyConn *cn, gboolean from_client, gchar *msg, gsize len)
         
     /* For pings we just write the same thing back */
     case SEAHORSE_SSH_PING_MSG:
-        write_ssh_message (cn->inchan, msg, len);
+        write_ssh_message (cn->inchan, (gchar*)msg, len);
         return FALSE;
     
-    case SSH_AGENTC_REQUEST_RSA_IDENTITIES:
-        version1 = TRUE;
-        /* Fall through */;
     case SSH2_AGENTC_REQUEST_IDENTITIES:
         break;
     
+    /* Keep our internal representation of what's cached consistent */
+    case SSH2_AGENTC_ADD_IDENTITY:
+    case SSH2_AGENTC_REMOVE_IDENTITY:
+    case SSH2_AGENTC_REMOVE_ALL_IDENTITIES:
+        clear_cached_fingerprints ();
+        g_timeout_add (500, update_status, NULL);
+        return TRUE;
+    
     /* Ignore all other messages */
     default:
         return TRUE;
@@ -342,11 +401,11 @@ process_message (SSHProxyConn *cn, gboolean from_client, gchar *msg, gsize len)
         return TRUE;
 
     /* If keys are already loaded then return */
-    count = get_num_identities (cn, version1);
+    count = get_num_identities (cn);
     if (count != 0)
         return TRUE;
     
-    keys = find_ssh_keys (version1);
+    keys = find_ssh_keys ();
     
     for (l = keys; l; l = g_list_next (l))
         load_ssh_key (SEAHORSE_SSH_KEY (l->data));
@@ -362,9 +421,9 @@ static gboolean
 io_handler (GIOChannel *source, GIOCondition condition, SSHProxyConn *cn)
 {
     GIOChannel *out;
-    gchar msg[1024];
+    gchar *msg;
     gboolean ret = FALSE;
-    gint length;
+    gsize length;
     gboolean from_client = FALSE;
     
     memset (msg, 0, sizeof (msg));
@@ -384,9 +443,9 @@ io_handler (GIOChannel *source, GIOCondition condition, SSHProxyConn *cn)
         else
             g_return_val_if_reached (FALSE);
         
-        length = read_ssh_message (source, msg, sizeof (msg));
+        msg = read_ssh_message (source, &length);
         
-        if (length <= 0) {
+        if (!msg) {
             free_connection (cn);
             cn = NULL;
             goto finally;
@@ -396,6 +455,7 @@ io_handler (GIOChannel *source, GIOCondition condition, SSHProxyConn *cn)
         if (process_message (cn, from_client, msg, length))
             write_ssh_message (out, msg, length);
 
+        g_free (msg);
         ret = TRUE;
     }
 
@@ -542,6 +602,9 @@ seahorse_agent_ssh_init ()
 
     /* All nicely done */
     ssh_agent_initialized = TRUE;
+    
+    /* We want any of our accesses to the agent to go directly: */
+    g_setenv ("SSH_AUTH_SOCK", ssh_client_sockname, TRUE);
     return TRUE;
 }
 
@@ -574,5 +637,68 @@ seahorse_agent_ssh_uninit ()
     if (ssh_agent_sockname[0] && ssh_client_sockname[0])
         g_rename (ssh_client_sockname, ssh_agent_sockname);
     
+    /* Free the cached data */
+    clear_cached_fingerprints ();
+    
     ssh_agent_initialized = FALSE;
 }
+
+GList*   
+seahorse_agent_ssh_cached_keys ()
+{
+    GList *keys = NULL;
+    SeahorseKey *skey;
+    SeahorseKeySource *sksrc;
+    GSList *l;
+    GQuark keyid;
+    
+    load_cached_fingerprints ();
+    
+    for (l = ssh_agent_cached_fingerprints; l; l = g_slist_next (l)) {
+        
+        keyid = seahorse_ssh_key_get_cannonical_id ((const gchar*)l->data);
+        g_return_val_if_fail (keyid, NULL);
+        
+        skey = seahorse_context_find_key (SCTX_APP (), keyid, SKEY_LOC_LOCAL);
+        if (!skey) {
+            sksrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_SSH, SKEY_LOC_UNKNOWN);
+            g_return_val_if_fail (sksrc != NULL, NULL);
+            skey = seahorse_unknown_source_add_key (SEAHORSE_UNKNOWN_SOURCE (sksrc), keyid, NULL);
+        }
+        
+        if (skey)
+            keys = g_list_prepend (keys, skey);
+    }
+    
+    return keys;
+}
+
+guint    
+seahorse_agent_ssh_count_keys ()
+{
+    load_cached_fingerprints ();
+    return g_slist_length (ssh_agent_cached_fingerprints);
+}
+
+void
+seahorse_agent_ssh_clearall ()
+{
+    SeahorseKeySource *sksrc;
+    GError *err = NULL;
+    gchar *output;
+
+    sksrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_SSH, SKEY_LOC_LOCAL);
+    g_return_if_fail (SEAHORSE_IS_SSH_SOURCE (sksrc));
+    
+    /* TODO: This could possibly done more efficiently by talking with the agent ourselves */
+    output = seahorse_ssh_operation_sync (SEAHORSE_SSH_SOURCE (sksrc), SSH_ADD_PATH " -D", &err);
+    if (!output) {
+        g_warning ("couldn't clear keys in SSH agent: %s", err && err->message ? err->message : "");
+        return;
+    }
+
+    clear_cached_fingerprints ();
+    update_status (NULL);
+    
+    g_free (output);
+}
