@@ -847,7 +847,7 @@ refresh_keyservers (GConfClient *client, guint id, GConfEntry *entry,
 
 SeahorseOperation*
 seahorse_context_transfer_keys (SeahorseContext *sctx, GList *keys, 
-                                SeahorseKeySource *to, gboolean individually)
+                                SeahorseKeySource *to)
 {
     SeahorseKeySource *from;
     SeahorseOperation *op = NULL;
@@ -864,13 +864,8 @@ seahorse_context_transfer_keys (SeahorseContext *sctx, GList *keys,
     
     while (keys) {
         
-        /* When individual, break off each one */
-        if (individually)
-            next = g_list_remove_link (keys, keys);
-     
-        /* Otherwise break off one set (same keysource) */
-        else 
-            next = seahorse_util_keylist_splice (keys);
+        /* break off one set (same keysource) */
+        next = seahorse_util_keylist_splice (keys);
 
         g_assert (SEAHORSE_IS_KEY (keys->data));
         skey = SEAHORSE_KEY (keys->data);
@@ -1034,7 +1029,7 @@ seahorse_context_discover_keys (SeahorseContext *sctx, GQuark ktype,
     
     /* Start an import process on all toimport */
     if (toimport) {
-        op = seahorse_context_transfer_keys (sctx, toimport, NULL, FALSE);
+        op = seahorse_context_transfer_keys (sctx, toimport, NULL);
         
         g_list_free (toimport);
         
