@@ -103,21 +103,31 @@ changed_key (SeahorseSSHKey *skey)
     key->ktype = SKEY_SSH;
     key->keyid = 0;
     
-    if (!skey->keydata || !skey->keydata->keyid) {
+    if (!skey->keydata || !skey->keydata->fingerprint) {
         
         key->location = SKEY_LOC_INVALID;
         key->etype = SKEY_ETYPE_NONE;
         key->loaded = SKEY_INFO_NONE;
         key->flags = SKEY_FLAG_DISABLED;
+        key->keydesc = _("Invalid");
+        key->rawid = NULL;
         
     } else {
     
         /* The key id */
-        key->keyid = seahorse_ssh_key_get_cannonical_id (skey->keydata->keyid);
+        key->keyid = seahorse_ssh_key_get_cannonical_id (skey->keydata->fingerprint);
+        key->rawid = skey->keydata->fingerprint;
         key->location = SKEY_LOC_LOCAL;
-        key->etype = skey->keydata->privfile ? SKEY_PRIVATE : SKEY_PUBLIC;
         key->loaded = SKEY_INFO_COMPLETE;
         key->flags = skey->keydata->authorized ? SKEY_FLAG_TRUSTED : 0;
+        
+        if (skey->keydata->privfile) {
+            key->etype = SKEY_PRIVATE;
+            key->keydesc = _("Private SSH Key");
+        } else {
+            key->etype = SKEY_PUBLIC;
+            key->keydesc = _("Public SSH Key");
+        }
     }
     
     if (!key->keyid)
@@ -172,6 +182,13 @@ seahorse_ssh_key_get_name_cn (SeahorseKey *skey, guint index)
     return NULL;
 }
 
+static SeahorseValidity  
+seahorse_ssh_key_get_name_validity  (SeahorseKey *skey, guint index)
+{
+    g_return_val_if_fail (index == 0, SEAHORSE_VALIDITY_UNKNOWN);
+    return calc_validity (SEAHORSE_SSH_KEY (skey));
+}
+
 static void
 seahorse_ssh_key_get_property (GObject *object, guint prop_id,
                                GValue *value, GParamSpec *pspec)
@@ -270,6 +287,7 @@ seahorse_ssh_key_class_init (SeahorseSSHKeyClass *klass)
     key_class->get_num_names = seahorse_ssh_key_get_num_names;
     key_class->get_name = seahorse_ssh_key_get_name;
     key_class->get_name_cn = seahorse_ssh_key_get_name_cn;
+    key_class->get_name_validity = seahorse_ssh_key_get_name_validity;
     
     g_object_class_install_property (gobject_class, PROP_KEY_DATA,
         g_param_spec_pointer ("key-data", "SSH Key Data", "SSH key data pointer",
@@ -374,28 +392,24 @@ seahorse_ssh_key_get_location (SeahorseSSHKey *skey)
 GQuark
 seahorse_ssh_key_get_cannonical_id (const gchar *id)
 {
-    static guint x = strlen (SKEY_SSH_STR);
-    gchar *p, *hex;
-    guint l, off = 0;
+    #define SSH_ID_SIZE 16
+    gchar *hex, *canonical_id = g_malloc0 (SSH_ID_SIZE + 1);
+    gint i, off, len = strlen (id);
     GQuark ret;
 
-    hex = g_strdup_printf ("%s:%s", SKEY_SSH_STR, id);
-    
-    /* Strip out all non alpha numeric chars */
-    for(p = hex + x + 1; *p; p++) {
-        if (g_ascii_isalnum(*p))
-            p[0 - off] = p[0];
-        else
-            off++;
+    /* Strip out all non alpha numeric chars and limit length to SSH_ID_SIZE */
+    for (i = len, off = SSH_ID_SIZE; i >= 0 && off > 0; --i) {
+         if (g_ascii_isalnum (id[i]))
+             canonical_id[--off] = g_ascii_toupper (id[i]);
     }
-    p[0 - off] = 0;
-    
-    /* Shorten where necessary */
-    l = strlen (hex);
-    if (l > (8 + x + 1))
-        hex[8 + x + 1] = 0;
     
+    /* Not enough characters */
+    g_return_val_if_fail (off == 0, 0);
+
+    hex = g_strdup_printf ("%s:%s", SKEY_SSH_STR, canonical_id);
     ret = g_quark_from_string (hex);
+    
+    g_free (canonical_id);
     g_free (hex);
     
     return ret;
