@@ -275,7 +275,7 @@ names_update_row (SeahorseKeyModel *skmodel, SeahorseKey *skey,
 
     icon = seahorse_key_get_location (skey) < SKEY_LOC_LOCAL ? 
                 GTK_STOCK_DIALOG_QUESTION : SEAHORSE_STOCK_SIGN;
-    name = seahorse_key_get_display_name (skey);
+    name = seahorse_key_get_name_markup (skey, 0);
     
     gtk_tree_store_set (GTK_TREE_STORE (skmodel), iter,
                         UIDSIG_INDEX, -1,
@@ -304,7 +304,7 @@ names_populate (SeahorseWidget *swidget, GtkTreeStore *store, SeahorsePGPKey *pk
     /* Insert all the fun-ness */
     for (i = 1, uid = pkey->pubkey->uids; uid; uid = uid->next, i++) {
 
-        name = seahorse_pgp_key_get_userid (pkey, i - 1);
+        name = seahorse_key_get_name_markup (SEAHORSE_KEY (pkey), i - 1);
 
         gtk_tree_store_append (store, &uiditer, NULL);
         gtk_tree_store_set (store, &uiditer,  
@@ -368,18 +368,20 @@ do_names (SeahorseWidget *swidget)
         /* This is our first time so create a store */
         store = GTK_TREE_STORE (seahorse_key_model_new (UIDSIG_N_COLUMNS, (GType*)uidsig_columns));
         g_signal_connect (store, "update-row", G_CALLBACK (names_update_row), swidget);
-
-        /* Make the columns for the view */
+        
+        /* Icon column */
         renderer = gtk_cell_renderer_pixbuf_new ();
         g_object_set (renderer, "stock-size", GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
         gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (widget),
                                                      -1, "", renderer,
                                                      "stock-id", UIDSIG_ICON, NULL);
 
+        /* The name column */
         gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (widget), 
                                                      -1, _("Name/Email"), gtk_cell_renderer_text_new (), 
-                                                     "text", UIDSIG_NAME, NULL);
+                                                     "markup", UIDSIG_NAME, NULL);
 
+        /* The signature ID column */
         gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (widget), 
                                                      -1, _("Signature ID"), gtk_cell_renderer_text_new (), 
                                                      "text", UIDSIG_KEYID, NULL);
@@ -578,7 +580,7 @@ set_photoid_state(SeahorseWidget *swidget, SeahorsePGPKey *pkey)
     if (photo_image) {
         if (!photoid || !photoid->photo)
             gtk_image_set_from_stock (GTK_IMAGE (photo_image), 
-                                      SEAHORSE_STOCK_PERSON, (GtkIconSize)-1);
+                etype == SKEY_PRIVATE ? SEAHORSE_STOCK_SECRET : SEAHORSE_STOCK_KEY, (GtkIconSize)-1);
         else 
             gtk_image_set_from_pixbuf (GTK_IMAGE (photo_image), photoid->photo);
     }
@@ -663,9 +665,7 @@ photoid_button_pressed(GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidg
 enum {
     UID_INDEX,
     UID_ICON,
-    UID_NAME,
-    UID_EMAIL,
-    UID_COMMENT,
+    UID_MARKUP,
     UID_N_COLUMNS
 };
 
@@ -760,7 +760,7 @@ do_owner (SeahorseWidget *swidget)
     gchar *text, *t;
     gulong expires_date;
     guint flags;
-    gchar *name, *email, *comment;
+    gchar *markup;
     int i;
 
     skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
@@ -846,35 +846,20 @@ do_owner (SeahorseWidget *swidget)
 
             gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (widget), 
                                                          -1, _("Name"), gtk_cell_renderer_text_new (), 
-                                                         "text", UID_NAME, NULL);
-    
-            gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (widget),
-                                                         -1, _("Email"), gtk_cell_renderer_text_new (), 
-                                                         "text", UID_EMAIL, NULL);
-    
-            gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (widget),
-                                                         -1, _("Comment"), gtk_cell_renderer_text_new (), 
-                                                         "text", UID_COMMENT, NULL);
+                                                         "markup", UID_MARKUP, NULL);
         }
     
         for (i = 1, uid = pkey->pubkey->uids; uid; uid = uid->next, i++) {
     
-            name = seahorse_pgp_key_get_userid_name (pkey, i - 1);
-            email = seahorse_pgp_key_get_userid_email (pkey, i - 1);
-            comment = seahorse_pgp_key_get_userid_comment (pkey, i - 1);
+            markup = seahorse_key_get_name_markup (SEAHORSE_KEY (pkey), i - 1);
     
             gtk_list_store_append (store, &iter);
             gtk_list_store_set (store, &iter,  
                                 UID_INDEX, i,
                                 UID_ICON, SEAHORSE_STOCK_PERSON,
-                                UID_NAME, name,
-                                UID_EMAIL, email,
-                                UID_COMMENT, comment,
-                                -1);
+                                UID_MARKUP, markup, -1);
             
-            g_free (name);
-            g_free (email);
-            g_free (comment);
+            g_free (markup);
         } 
         
         gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL(store));
@@ -909,6 +894,18 @@ const GType subkey_columns[] = {
     G_TYPE_STRING   /* length*/
 };
 
+/* trust combo box list */
+enum {
+    TRUST_LABEL,
+    TRUST_VALIDITY,
+    TRUST_N_COLUMNS
+};
+
+const GType trust_columns[] = {
+    G_TYPE_STRING,  /* label */
+    G_TYPE_INT      /* validity */
+};
+
 static gint 
 get_selected_subkey (SeahorseWidget *swidget)
 {
@@ -948,39 +945,29 @@ trust_changed (GtkComboBox *selection, SeahorseWidget *swidget)
 {
 	SeahorseKey *skey;
 	gint trust;
-	SeahorseValidity trust_enum;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
 	gpgme_error_t err;
+	gboolean set;
 	
-	trust = gtk_combo_box_get_active (selection);
+	set = gtk_combo_box_get_active_iter (selection, &iter);
+	g_assert (set);
 	
-	switch (trust) {
-	    case 0:
-	        trust_enum = SEAHORSE_VALIDITY_UNKNOWN;
-	        break;
-        case 1:
-            trust_enum = SEAHORSE_VALIDITY_NEVER;
-            break;
-        case 2:
-            trust_enum = SEAHORSE_VALIDITY_MARGINAL;
-            break;
-        case 3:
-            trust_enum = SEAHORSE_VALIDITY_FULL;
-            break;
-        case 4:
-            trust_enum = SEAHORSE_VALIDITY_ULTIMATE;
-            break;
-        default:
-            trust_enum = SEAHORSE_VALIDITY_UNKNOWN;
+	if (set) {
+    	skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    	
+    	model = gtk_combo_box_get_model (selection);
+    	gtk_tree_model_get (model, &iter,
+                            TRUST_VALIDITY, &trust,
+                            -1);
+                                  
+    	if (seahorse_key_get_trust (skey) != trust) {
+    		err = seahorse_pgp_key_op_set_trust (SEAHORSE_PGP_KEY (skey), trust);
+    		if (err) {
+    			seahorse_util_handle_gpgme (err, _("Unable to change trust"));
+    		}
+    	}
     }
-	
-	skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
-	
-	if (seahorse_key_get_trust (skey) != trust_enum) {
-		err = seahorse_pgp_key_op_set_trust (SEAHORSE_PGP_KEY (skey), trust_enum);
-		if (err) {
-			seahorse_util_handle_gpgme (err, _("Unable to change trust"));
-		}
-	}
 }
 
 static void
@@ -1019,7 +1006,7 @@ details_export_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
         gpgmex_data_release (data);
         
         if (!seahorse_operation_is_successful (op))
-            seahorse_operation_steal_error (op, &err);
+            seahorse_operation_copy_error (op, &err);
     }
     
     if (err)
@@ -1052,6 +1039,71 @@ details_calendar_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
 	seahorse_expires_new (SEAHORSE_PGP_KEY (skey), index);	
 }
 
+static void
+setup_details_trust (SeahorseWidget *swidget)
+{
+    SeahorseKey *skey;
+    SeahorseKeyEType etype;
+    GtkWidget *widget;
+    GtkListStore *model;
+    GtkTreeIter iter;
+    GtkCellRenderer *text_cell = gtk_cell_renderer_text_new ();
+
+    DBG_PRINT(("KeyProperties: Setting up Trust Combo Box Store\n"));
+    
+    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    etype = seahorse_key_get_etype (skey);
+    
+    widget = glade_xml_get_widget (swidget->xml, "details-trust-combobox");
+    
+    gtk_cell_layout_clear(GTK_CELL_LAYOUT (widget));
+    
+    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT (widget), text_cell, FALSE);
+    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT (widget), text_cell,
+                                   "text", TRUST_LABEL,
+                                   NULL);
+    
+    /* Initialize the store */
+    model = gtk_list_store_newv (TRUST_N_COLUMNS, (GType*)trust_columns);
+    
+    if (etype != SKEY_PRIVATE) {
+        gtk_list_store_append (model, &iter);
+        gtk_list_store_set (model, &iter,
+                            TRUST_LABEL, _("Unknown"),
+                            TRUST_VALIDITY,  SEAHORSE_VALIDITY_UNKNOWN,
+                            -1);                               
+       
+        gtk_list_store_append (model, &iter);
+        gtk_list_store_set (model, &iter,
+                            TRUST_LABEL, _("Never"),
+                            TRUST_VALIDITY,  SEAHORSE_VALIDITY_NEVER,
+                            -1);
+    }
+    
+    gtk_list_store_append (model, &iter);
+    gtk_list_store_set (model, &iter,
+                        TRUST_LABEL, _("Marginal"),
+                        TRUST_VALIDITY,  SEAHORSE_VALIDITY_MARGINAL,
+                        -1);
+    
+    gtk_list_store_append (model, &iter);
+    gtk_list_store_set (model, &iter,
+                        TRUST_LABEL, _("Full"),
+                        TRUST_VALIDITY,  SEAHORSE_VALIDITY_FULL,
+                        -1);
+    if (etype == SKEY_PRIVATE) {
+        gtk_list_store_append (model, &iter);
+        gtk_list_store_set (model, &iter,
+                            TRUST_LABEL, _("Ultimate"),
+                            TRUST_VALIDITY,  SEAHORSE_VALIDITY_ULTIMATE,
+                            -1);
+    }
+    
+    gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (model));                                        
+
+    DBG_PRINT(("KeyProperties: Finished Setting up Trust Combo Box Store\n"));
+}
+
 static void
 do_details_signals (SeahorseWidget *swidget) 
 { 
@@ -1100,6 +1152,7 @@ do_details (SeahorseWidget *swidget)
     gpgme_subkey_t subkey;
     GtkWidget *widget;
     GtkListStore *store;
+    GtkTreeModel *model;
     GtkTreeIter iter;
     GtkTreeIter default_key;
     GtkTreeSelection *selection;
@@ -1107,8 +1160,8 @@ do_details (SeahorseWidget *swidget)
     gchar *fp_label, *expiration_date, *created_date;
     const gchar *label, *status, *length;
     gint subkey_number, key, trust;
-    SeahorseValidity trust_enum;
     guint keyloc;
+    gboolean valid;
 
     skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
     pkey = SEAHORSE_PGP_KEY (skey);
@@ -1161,32 +1214,25 @@ do_details (SeahorseWidget *swidget)
 
     show_glade_widget (swidget, "details-trust-combobox", keyloc == SKEY_LOC_LOCAL);
     widget = glade_xml_get_widget (swidget->xml, "details-trust-combobox");
+    
     if (widget) {
         gtk_widget_set_sensitive (widget, !(pkey->pubkey->disabled));
-        trust_enum = seahorse_key_get_trust (skey);
+        model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
         
-        switch (trust_enum) {
-            case SEAHORSE_VALIDITY_UNKNOWN:
-                trust = 0;
+        valid = gtk_tree_model_get_iter_first (model, &iter);
+       
+        while (valid) {
+            gtk_tree_model_get (model, &iter,
+                                TRUST_VALIDITY, &trust,
+                                -1);
+            
+            if (trust == seahorse_key_get_trust (skey)) {
+                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget),  &iter);
                 break;
-            case SEAHORSE_VALIDITY_NEVER:
-                trust = 1;
-                break;
-            case SEAHORSE_VALIDITY_MARGINAL:
-                trust = 2;
-                break;
-            case SEAHORSE_VALIDITY_FULL:
-                trust = 3;
-                break;
-            case SEAHORSE_VALIDITY_ULTIMATE:
-                trust = 4;
-                break;
-            default:
-                trust = 0;
+            }
+            
+            valid = gtk_tree_model_iter_next (model, &iter);             
         }
-        
-        if (0 < trust)
-            gtk_combo_box_set_active (GTK_COMBO_BOX (widget),  trust);
     }
 
     /* Clear/create table store */
@@ -1743,6 +1789,7 @@ setup_public_properties (SeahorsePGPKey *pkey)
     do_owner (swidget);
     do_owner_signals (swidget);
 
+    setup_details_trust (swidget);
     do_details (swidget);
     do_details_signals (swidget);
 
@@ -1782,6 +1829,7 @@ setup_private_properties (SeahorsePGPKey *pkey)
     do_names (swidget);
     do_names_signals (swidget);
 
+    setup_details_trust (swidget);
     do_details (swidget);
     do_details_signals (swidget);
 
